﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CG2.Modeling;
using CG2.Rendering;
using CG2.Mathematics;
using CG2.Shading;
using CG2.Lighting;
using CG2.Texturing;

namespace CG2.Ex07
{
    /***********************************************************************
    What's new?
    ***********************************************************************/
    public partial class Form1 : Form
    {
        public static Random Rnd = new Random(0);
        Camera camera;

        public Form1()
        {
            InitializeComponent();
            cbScene.SelectedIndex = 0;
            cbAntiAlias.SelectedIndex = 0;
        }

        public void InitSceneSampling()
        {
            World world = new World();

            #region Shaders

            Shader homerX0 = new PhongShader()
            {
                DiffuseSampler = new Texture2Sampler("Textures/homer.png", null, TextureFiltering.Nearest, TextureWrap.Mirror, TextureWrap.Mirror)
                {
                    Mapping = new PlaneMapping(new Vector4(0, 0, 0), 2 * Vector4.UnitY, 2 * Vector4.UnitZ)
                }
            };

            Shader homerY0 = new PhongShader()
            {
                DiffuseSampler = new Texture2Sampler("Textures/homer.png", null, TextureFiltering.Nearest, TextureWrap.Clamp, TextureWrap.Clamp)
                {
                    Mapping = new PlaneMapping(new Vector4(0, 0, 0), 15 * Vector4.UnitX, 15 * Vector4.UnitZ)
                }
            };

            Shader homerZ0 = new PhongShader()
            {
                DiffuseSampler = new Texture2Sampler("Textures/homer.png", null, TextureFiltering.Nearest, TextureWrap.Mirror, TextureWrap.Repeat)
                {
                    Mapping = new PlaneMapping(new Vector4(0, 0, 0), 3 * Vector4.UnitX, 3 * Vector4.UnitY)
                }
            };


            Shader homerY1 = new PhongShader()
            {
                DiffuseSampler = new Texture2Sampler("Textures/homer.png", null, TextureFiltering.Bilinear, TextureWrap.Clamp, TextureWrap.Clamp)
                {
                    Mapping = new PlaneMapping(new Vector4(0, 10, 0), 15 * Vector4.UnitX, 15 * Vector4.UnitZ)
                }
            };

            Shader homerZ1 = new PhongShader()
            {
                DiffuseSampler = new Texture2Sampler("Textures/homer.png", null, TextureFiltering.Nearest, TextureWrap.Clamp, TextureWrap.Clamp)
                {
                    Mapping = new PlaneMapping(new Vector4(0, 0, 8), 5 * Vector4.UnitX, 5 * Vector4.UnitY),
                }
            };

            #endregion

            #region Models

            world.Models.Add(new Plane(homerX0, new Vector4(0, 0, 0), new Vector4(1, 0, 0)));
            world.Models.Add(new Plane(homerY0, new Vector4(0, 0, 0), new Vector4(0, 1, 0)));
            world.Models.Add(new Plane(homerZ0, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));
            world.Models.Add(new Plane(homerY1, new Vector4(0, 10, 0), new Vector4(0, -1, 0)));
            world.Models.Add(new Plane(homerZ1, new Vector4(0, 0, 8), new Vector4(0, 0, -1)));

            #endregion

            #region Lights

            PointLight point = new PointLight()
            {
                Origin = new Vector4(7, 5, 6),
                AmbientColor = new Vector4(0.1, 0.1, 0.1),
                Intensity = 1.2,
                Range = 7.0,
            };
            world.Lights.Add(point);

            #endregion

            camera = new Camera(500, 400)
            {
                Position = new Vector4(20, 5, 4.5),
                Target = new Vector4(0, 5, 4.5),
                World = world,
            };
        }

        public void InitSceneMapping()
        {
            World world = new World();

            #region Shaders

            Shader red = new PhongShader(new Vector4(0.5, 0, 0));

            Shader checker = new PhongShader()
            {
                DiffuseSampler = new Texture2Sampler("Textures/checker.png", null, TextureFiltering.Nearest, TextureWrap.Repeat, TextureWrap.Repeat)
                {
                    Mapping = new SphereMapping(new Vector4(15, 3, 4), Vector4.UnitX, Vector4.UnitY) { Scale = new Vector4(0.4, 0.4, 1) }
                }
            };

            #endregion


            #region Models

            world.Models.Add(new Plane(red, new Vector4(0, 0, 0), new Vector4(0, 0, 1)));

            world.Models.Add(new Sphere(checker, new Vector4(15, 3, 4), 4));

            #endregion


            #region Lights

            PointLight point = new PointLight()
            {
                Origin = new Vector4(30, 0, 20),
                //SpecularColor = new Vector4(0,0,0), 
                AmbientColor = new Vector4(0.3, 0.3, 0.3),
                Intensity = 1.0,
                Range = 20.0,
                LinearAttenuation = 0.2,
                QuadraticAttenuation = 0
            };
            world.Lights.Add(point);

            #endregion

            camera = new Camera(500, 400)
            {
                Position = new Vector4(30, 5, 15),
                Target = new Vector4(10, 5, 2),
                World = world,
            };
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            Graphics g = e.Graphics;

            if (camera.Bitmap != null) g.DrawImage(camera.Bitmap, 0, 0);
        }

        private void bRender_Click(object sender, EventArgs e)
        {
            camera.UseShadows = cbUseShadows.Checked;
            Cursor = Cursors.WaitCursor;
            DateTime t0 = DateTime.Now;
            
            camera.Render();

            DateTime t1 = DateTime.Now;
            Cursor = Cursors.Default;

            lRenderTime.Text = "Rendering: " + (t1 - t0).TotalMilliseconds.ToString("F0") + " ms";

            Invalidate();
        }

        private void cbScene_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch ((String)cbScene.SelectedItem)
            {
                case "Sphere Mapping": InitSceneMapping(); break;
                case "Plane Mapping": InitSceneSampling(); break;
            }
        }

        private void cbAntiAlias_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch ((String)cbAntiAlias.SelectedItem)
            {
                case "No AntiAliasing": camera.SubSampleCount = 1; break;
                case "2x2 AntiAliasing": camera.SubSampleCount = 2; break;
                case "4x4 AntiAliasing": camera.SubSampleCount = 4; break;
                case "8x8 AntiAliasing": camera.SubSampleCount = 8; break;
            }   
        }


    }
}
