﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;
using CG2.Rendering;
using CG2.Shading;

namespace CG2.Modeling
{
    class Mesh : Model
    {   
        // ToDo: Finish the implementation of mesh class
        //       Each Mesh has its faces / Each face has its Vertices  
        public class Vertex
        {
            public Vector4 Position;

            public Vertex()
            {
            }

            public Vertex(Vector4 position)
            {
                Position = position;
            }

            public Vertex(Double x, Double y, Double z)
            {
                Position = new Vector4(x, y, z);
            }
        }

        public class Face
        {
            // Three vertices of the triangle
            public Vertex V0, V1, V2;
            // Collision helpers
            private Vector4 u, v, n;
            private Double uu, vv, uv, k;

            public void Set(
                Double P1, Double P2, Double P3,
                Double P4, Double P5, Double P6,
                Double P7, Double P8, Double P9)
            {
                // ToDo: set face from the given points
                V0 = new Vertex() { Position = new Vector4(P1, P2, P3) };
                V1 = new Vertex() { Position = new Vector4(P4, P5, P6) };
                V2 = new Vertex() { Position = new Vector4(P7, P8, P9) };

                // ToDo: calculate face collision info (f.e., - u, v, n, uu, vv, uv, k)
                u = V1.Position - V0.Position;
                v = V2.Position - V0.Position;
                uu = u * u;
                vv = v * v;
                uv = u * v;
                k = 1 / (uv * uv - uu * vv);
                n = (u % v).Normalized;
            }

            public void Colide(Ray ray, Mesh model)
            {
                // ToDo: collide ray with the face
                Double nv = n * ray.Direction;
                if (nv >= -0.00001 && nv <= 0.00001) return;

                // ToDo: Calculate parameter (derived from plane) and then test if it is inside of the triangle
                Double param = (n * (V0.Position - ray.Origin)) / nv;

                //ToDo: Collisions beyond current ray param are ignored
                if (param < Ray.Bias || param >= ray.HitParam) return; 

                Vector4 p = ray.Origin + param * ray.Direction;
                Vector4 w = p - V0.Position;
                Double wu = w * u;
                Double wv = w * v;
                Double s = k * (uv * wv - vv * wu);
                Double t = k * (uv * wu - uu * wv);

                // ToDo: Test if collision point p is out of the triangle
                if (s < 0 || t < 0 || s + t > 1) return; 

                // ToDo: If hit set HitModel, HitParam and HitNormal
                ray.HitModel = model;
                ray.HitParam = param;

                // ToDo: Yes, normal vector is also important
                ray.HitNormal = n;
            }
        }

        #region Properties
        
        //All faces of the mesh, prepared for rendering
        public List<Face> Faces = new List<Face>();

        //Loaded Data from .obj file
        ObjMesh Data;

        #endregion

        #region Init

        public Mesh(Shader shader, string fileName)
        {
            Data = new ObjMesh(fileName);
            Shader = shader;

            // ToDo: Fill all faces from the Data file
            SetFaces();
        }

        public void SetFaces()
        {
            Faces = new List<Face>();
            Int32 Ln = Data.Triangles.Length;
            for (Int32 i = 0; i < Ln; i++)
            {
                Face face = new Face();
                // ToDo: Set each face and its vertices
                face.Set(
                    Data.Vertices[Data.Triangles[i].Index0].Vertex.X,
                    Data.Vertices[Data.Triangles[i].Index0].Vertex.Y,
                    Data.Vertices[Data.Triangles[i].Index0].Vertex.Z,
                    Data.Vertices[Data.Triangles[i].Index1].Vertex.X,
                    Data.Vertices[Data.Triangles[i].Index1].Vertex.Y,
                    Data.Vertices[Data.Triangles[i].Index1].Vertex.Z,
                    Data.Vertices[Data.Triangles[i].Index2].Vertex.X,
                    Data.Vertices[Data.Triangles[i].Index2].Vertex.Y,
                    Data.Vertices[Data.Triangles[i].Index2].Vertex.Z);
                Faces.Add(face);
            }
        }

        #endregion

        #region Raytracing

        public override void Collide(Ray ray)
        {
            // ToDo: collide with each face of mesh
            foreach (Face face in Faces)
            {
                face.Colide(ray, this);
            }
        }

        #endregion
    }
}
