using System;
using System.Runtime.InteropServices;
using CG2.Mathematics;
using CG2.Rendering;

namespace CG2.Modeling
{
    public class ObjMesh
    {
        /************************************************************************
        This class is standard .obj loader. 
        It can load any .obj model, as long as triangulated and unwrapped
        See > http://goo.gl/xAiFC for more information about the format .obj
         
         Tutorial * How to create your own model suitable for loading from Blender to this application
         Blender   vs    CG2
          y++      ->    z--
          x++      ->    x++
          z++      ->    y++
          mesh must be made from triangles
          mesh must be unwrapped
          mesh must be exported with normals and uv tex-coords
        ************************************************************************/

        public ObjMesh(string fileName)
        {
            ObjMeshLoader.Load(this, fileName);
        }

        public ObjVertex[] Vertices
        {
            get { return vertices; }
            set { vertices = value; }
        }
        ObjVertex[] vertices;

        public ObjTriangle[] Triangles
        {
            get { return triangles; }
            set { triangles = value; }
        }
        ObjTriangle[] triangles;

        [StructLayout(LayoutKind.Sequential)]
        public struct ObjVertex
        {
            public Vector4 TexCoord;
            public Vector4 Normal;
            public Vector4 Vertex;
        }
        
        [StructLayout(LayoutKind.Sequential)]
        public class ObjTriangle
        {
            public int Index0;
            public int Index1;
            public int Index2;
        }
    }
}
