﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CG2.Mathematics;
using CG2.Modeling;

namespace CG2.Rendering
{
	public class Ray
	{
		#region Properties

        // Bias helps to overcome numerical precision issues resulting in false ray-primitive intersections
        // If the intersection is near the camera - neglected
        // Or to far from the camera - neglected
        // Bias is different to the ray tracing, shading, shadows etc. 
        // - understand as some ad-hoc helping constant
		public static Double Bias = 0.1;

		public Vector4 Origin;
		public Vector4 Direction;

		public Double HitParam = Double.MaxValue;
		public Vector4 HitNormal;
		public Model HitModel = null;

		#endregion

		#region Init

		public Ray()
		{
		}

		public Ray(Vector4 origin, Vector4 direction)
		{
			Set(origin, direction);
		}

		public void Set(Vector4 origin, Vector4 direction)
		{
			Origin = origin;
			Direction = direction;
			HitParam = Double.MaxValue;
		}

		#endregion

		#region Geometry

        public Vector4 GetHitPoint()
        {
            return Origin + HitParam * Direction;
        }

		#endregion
	}
}
